@@ -399,6 +399,7 @@ static void IPADDRESS_SetFocusToField (const IPADDRESS_INFO *infoPtr, INT index)
 
     if (index > 3 || index < 0) index=0;
 
+    SendMessageW (infoPtr->Part[index].EditHwnd, EM_SETSEL, 0, -1);
     SetFocus (infoPtr->Part[index].EditHwnd);
 }
 
@@ -637,6 +638,10 @@ IPADDRESS_WindowProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 	case IPM_ISBLANK:
 	    return IPADDRESS_IsBlank (infoPtr);
 
+        case WM_SETFOCUS:
+            IPADDRESS_SetFocusToField (infoPtr, 0);
+            break;
+
 	default:
 	    if ((uMsg >= WM_USER) && (uMsg < WM_APP) && !COMCTL32_IsReflectedMessage(uMsg))
 		ERR("unknown msg %04x wp=%08lx lp=%08lx\n", uMsg, wParam, lParam);
