@@ -43,8 +43,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(ntdll);
 
-#define NT_SUCCESS(status) (status == STATUS_SUCCESS)
-
 #define SELF_RELATIVE_FIELD(sd,field) ((BYTE *)(sd) + ((SECURITY_DESCRIPTOR_RELATIVE *)(sd))->field)
 
 /* helper function to retrieve active length of an ACL */
@@ -860,6 +858,7 @@ NTSTATUS WINAPI RtlMakeSelfRelativeSD(
         return STATUS_INVALID_PARAMETER;
 
     length = RtlLengthSecurityDescriptor(pAbs);
+    if (!(pAbs->Control & SE_SELF_RELATIVE)) length -= (sizeof(*pAbs) - sizeof(*pRel));
     if (*lpdwBufferLength < length)
     {
         *lpdwBufferLength = length;
@@ -1520,7 +1519,7 @@ RtlAdjustPrivilege(ULONG Privilege,
                                     &TokenHandle);
     }
 
-    if (!NT_SUCCESS(Status))
+    if (Status)
     {
         WARN("Retrieving token handle failed (Status %x)\n", Status);
         return Status;
@@ -1545,7 +1544,7 @@ RtlAdjustPrivilege(ULONG Privilege,
         TRACE("Failed to assign all privileges\n");
         return STATUS_PRIVILEGE_NOT_HELD;
     }
-    if (!NT_SUCCESS(Status))
+    if (Status)
     {
         WARN("NtAdjustPrivilegesToken() failed (Status %x)\n", Status);
         return Status;
@@ -1775,7 +1774,8 @@ NTSTATUS WINAPI NtSetSecurityObject(HANDLE Handle,
             return STATUS_INVALID_SECURITY_DESCR;
     }
 
-    if (SecurityInformation & SACL_SECURITY_INFORMATION)
+    if (SecurityInformation & SACL_SECURITY_INFORMATION ||
+        SecurityInformation & LABEL_SECURITY_INFORMATION)
     {
         status = RtlGetSaclSecurityDescriptor( SecurityDescriptor, &present, &sacl, &defaulted );
         if (status != STATUS_SUCCESS) return status;
@@ -1922,3 +1922,14 @@ BOOL WINAPI RtlConvertToAutoInheritSecurityObject(
 
     return FALSE;
 }
+
+/******************************************************************************
+ * RtlDefaultNpAcl (NTDLL.@)
+ */
+NTSTATUS WINAPI RtlDefaultNpAcl(PACL *pAcl)
+{
+    FIXME("%p - stub\n", pAcl);
+
+    *pAcl = NULL;
+    return STATUS_SUCCESS;
+}
