@@ -68,6 +68,9 @@ static const WCHAR odbc_error_load_lib_failed[] = {'L','o','a','d',' ','L','i','
 static const WCHAR odbc_error_request_failed[] = {'R','e','q','u','e','s','t',' ','F','a','i','l','e','d',0};
 static const WCHAR odbc_error_invalid_keyword[] = {'I','n','v','a','l','i','d',' ','k','e','y','w','o','r','d',' ','v','a','l','u','e',0};
 
+static BOOL (WINAPI *pConfigDSN)(HWND hwnd, WORD request, const char *driver, const char *attr);
+static BOOL (WINAPI *pConfigDSNW)(HWND hwnd, WORD request, const WCHAR *driver, const WCHAR *attr);
+
 /* Push an error onto the error stack, taking care of ranges etc. */
 static void push_error(int code, LPCWSTR msg)
 {
@@ -219,30 +222,6 @@ static BOOL SQLInstall_narrow(int mode, LPSTR buffer, LPCWSTR str, WORD str_leng
     return success;
 }
 
-BOOL WINAPI SQLConfigDataSourceW(HWND hwndParent, WORD fRequest,
-               LPCWSTR lpszDriver, LPCWSTR lpszAttributes)
-{
-    LPCWSTR p;
-
-    clear_errors();
-    FIXME("%p %d %s %s\n", hwndParent, fRequest, debugstr_w(lpszDriver),
-          debugstr_w(lpszAttributes));
-
-    for (p = lpszAttributes; *p; p += lstrlenW(p) + 1)
-        FIXME("%s\n", debugstr_w(p));
-
-    return TRUE;
-}
-
-BOOL WINAPI SQLConfigDataSource(HWND hwndParent, WORD fRequest,
-               LPCSTR lpszDriver, LPCSTR lpszAttributes)
-{
-    FIXME("%p %d %s %s\n", hwndParent, fRequest, debugstr_a(lpszDriver),
-          debugstr_a(lpszAttributes));
-    clear_errors();
-    return TRUE;
-}
-
 static HMODULE load_config_driver(const WCHAR *driver)
 {
     static WCHAR reg_driver[] = {'d','r','i','v','e','r',0};
@@ -288,7 +267,7 @@ static HMODULE load_config_driver(const WCHAR *driver)
     if(ret != ERROR_SUCCESS)
     {
         HeapFree(GetProcessHeap(), 0, filename);
-        push_error(ODBC_ERROR_INVALID_DSN, odbc_error_invalid_dsn);
+        push_error(ODBC_ERROR_COMPONENT_NOT_FOUND, odbc_error_component_not_found);
         return NULL;
     }
 
@@ -359,6 +338,148 @@ fail:
     return FALSE;
 }
 
+static WORD map_request(WORD request)
+{
+    switch (request)
+    {
+    case ODBC_ADD_DSN:
+    case ODBC_ADD_SYS_DSN:
+        return ODBC_ADD_DSN;
+
+    case ODBC_CONFIG_DSN:
+    case ODBC_CONFIG_SYS_DSN:
+        return ODBC_CONFIG_DSN;
+
+    case ODBC_REMOVE_DSN:
+    case ODBC_REMOVE_SYS_DSN:
+        return ODBC_REMOVE_DSN;
+
+    default:
+        FIXME("unhandled request %u\n", request);
+        return 0;
+    }
+}
+
+static UWORD get_config_mode(WORD request)
+{
+    if (request == ODBC_ADD_DSN || request == ODBC_CONFIG_DSN || request == ODBC_REMOVE_DSN) return ODBC_USER_DSN;
+    return ODBC_SYSTEM_DSN;
+}
+
+BOOL WINAPI SQLConfigDataSourceW(HWND hwnd, WORD request, LPCWSTR driver, LPCWSTR attributes)
+{
+    HMODULE mod;
+    BOOL ret = FALSE;
+    UWORD config_mode_prev = config_mode;
+    WORD mapped_request;
+
+    TRACE("%p, %d, %s, %s\n", hwnd, request, debugstr_w(driver), debugstr_w(attributes));
+    if (TRACE_ON(odbc))
+    {
+        const WCHAR *p;
+        for (p = attributes; *p; p += lstrlenW(p) + 1)
+            TRACE("%s\n", debugstr_w(p));
+    }
+
+    clear_errors();
+
+    mapped_request = map_request(request);
+    if (!mapped_request)
+        return FALSE;
+
+    mod = load_config_driver(driver);
+    if (!mod)
+        return FALSE;
+
+    config_mode = get_config_mode(request);
+
+    pConfigDSNW = (void*)GetProcAddress(mod, "ConfigDSNW");
+    if(pConfigDSNW)
+        ret = pConfigDSNW(hwnd, mapped_request, driver, attributes);
+    else
+        ERR("Failed to find ConfigDSNW\n");
+
+    config_mode = config_mode_prev;
+
+    if (!ret)
+        push_error(ODBC_ERROR_REQUEST_FAILED, odbc_error_request_failed);
+
+    FreeLibrary(mod);
+
+    return ret;
+}
+
+BOOL WINAPI SQLConfigDataSource(HWND hwnd, WORD request, LPCSTR driver, LPCSTR attributes)
+{
+    HMODULE mod;
+    BOOL ret = FALSE;
+    WCHAR *driverW;
+    UWORD config_mode_prev = config_mode;
+    WORD mapped_request;
+
+    TRACE("%p, %d, %s, %s\n", hwnd, request, debugstr_a(driver), debugstr_a(attributes));
+
+    if (TRACE_ON(odbc))
+    {
+        const char *p;
+        for (p = attributes; *p; p += lstrlenA(p) + 1)
+            TRACE("%s\n", debugstr_a(p));
+    }
+
+    clear_errors();
+
+    mapped_request = map_request(request);
+    if (!mapped_request)
+        return FALSE;
+
+    driverW = heap_strdupAtoW(driver);
+    if (!driverW)
+    {
+        push_error(ODBC_ERROR_OUT_OF_MEM, odbc_error_out_of_mem);
+        return FALSE;
+    }
+
+    mod = load_config_driver(driverW);
+    if (!mod)
+    {
+        heap_free(driverW);
+        return FALSE;
+    }
+
+    config_mode = get_config_mode(request);
+
+    pConfigDSN = (void*)GetProcAddress(mod, "ConfigDSN");
+    if (pConfigDSN)
+    {
+        TRACE("Calling ConfigDSN\n");
+        ret = pConfigDSN(hwnd, mapped_request, driver, attributes);
+    }
+    else
+    {
+        pConfigDSNW = (void*)GetProcAddress(mod, "ConfigDSNW");
+        if (pConfigDSNW)
+        {
+            WCHAR *attr = NULL;
+            TRACE("Calling ConfigDSNW\n");
+
+            attr = SQLInstall_strdup_multi(attributes);
+            if(attr)
+                ret = pConfigDSNW(hwnd, mapped_request, driverW, attr);
+            heap_free(attr);
+        }
+    }
+
+    config_mode = config_mode_prev;
+
+    if (!ret)
+        push_error(ODBC_ERROR_REQUEST_FAILED, odbc_error_request_failed);
+
+    heap_free(driverW);
+    FreeLibrary(mod);
+
+    return ret;
+}
+
 BOOL WINAPI SQLConfigDriverW(HWND hwnd, WORD request, LPCWSTR driver,
                LPCWSTR args, LPWSTR msg, WORD msgmax, WORD *msgout)
 {
@@ -583,7 +704,8 @@ BOOL WINAPI SQLGetInstalledDrivers(char *buf, WORD size, WORD *sizeout)
         return FALSE;
     }
 
-    *sizeout = WideCharToMultiByte(CP_ACP, 0, wbuf, written, NULL, 0, NULL, NULL);
+    if (sizeout)
+        *sizeout = WideCharToMultiByte(CP_ACP, 0, wbuf, written, NULL, 0, NULL, NULL);
     WideCharToMultiByte(CP_ACP, 0, wbuf, written, buf, size, NULL, NULL);
 
     heap_free(wbuf);
