@@ -1884,6 +1884,7 @@ static HRESULT PropertyStorage_WritePropertyToStream(PropertyStorage_impl *This,
     HRESULT hr;
     LARGE_INTEGER seek;
     PROPERTYIDOFFSET propIdOffset;
+    ULARGE_INTEGER ularge;
     ULONG count;
     DWORD dwType, bytesWritten;
 
@@ -1945,6 +1946,13 @@ static HRESULT PropertyStorage_WritePropertyToStream(PropertyStorage_impl *This,
         bytesWritten = count;
         break;
     }
+    case VT_I8:
+    case VT_UI8:
+    {
+        StorageUtl_WriteULargeInteger(&ularge, 0, &var->u.uhVal);
+        hr = IStream_Write(This->stm, &ularge, sizeof(ularge), &bytesWritten);
+        break;
+    }
     case VT_LPSTR:
     {
         DWORD len, dwTemp;
