@@ -28,9 +28,6 @@
 #include "wine/windef16.h"
 #include "winbase.h"
 #include "wingdi.h"
-#include "winuser.h"
-#include "winnls.h"
-#include "winternl.h"
 #include "controls.h"
 #include "user_private.h"
 
@@ -243,72 +240,6 @@ DWORD WINAPI SetLogonNotifyWindow(HWINSTA hwinsta,HWND hwnd)
     return 1;
 }
 
-static const WCHAR primary_device_name[] = {'\\','\\','.','\\','D','I','S','P','L','A','Y','1',0};
-static const WCHAR primary_device_string[] = {'X','1','1',' ','W','i','n','d','o','w','i','n','g',' ',
-                                              'S','y','s','t','e','m',0};
-static const WCHAR primary_device_deviceid[] = {'P','C','I','\\','V','E','N','_','0','0','0','0','&',
-                                                'D','E','V','_','0','0','0','0',0};
-
-/***********************************************************************
- *		EnumDisplayDevicesA (USER32.@)
- */
-BOOL WINAPI EnumDisplayDevicesA( LPCSTR lpDevice, DWORD i, LPDISPLAY_DEVICEA lpDispDev,
-                                 DWORD dwFlags )
-{
-    UNICODE_STRING deviceW;
-    DISPLAY_DEVICEW ddW;
-    BOOL ret;
-
-    if(lpDevice)
-        RtlCreateUnicodeStringFromAsciiz(&deviceW, lpDevice); 
-    else
-        deviceW.Buffer = NULL;
-
-    ddW.cb = sizeof(ddW);
-    ret = EnumDisplayDevicesW(deviceW.Buffer, i, &ddW, dwFlags);
-    RtlFreeUnicodeString(&deviceW);
-
-    if(!ret) return ret;
-
-    WideCharToMultiByte(CP_ACP, 0, ddW.DeviceName, -1, lpDispDev->DeviceName, sizeof(lpDispDev->DeviceName), NULL, NULL);
-    WideCharToMultiByte(CP_ACP, 0, ddW.DeviceString, -1, lpDispDev->DeviceString, sizeof(lpDispDev->DeviceString), NULL, NULL);
-    lpDispDev->StateFlags = ddW.StateFlags;
-
-    if(lpDispDev->cb >= offsetof(DISPLAY_DEVICEA, DeviceID) + sizeof(lpDispDev->DeviceID))
-        WideCharToMultiByte(CP_ACP, 0, ddW.DeviceID, -1, lpDispDev->DeviceID, sizeof(lpDispDev->DeviceID), NULL, NULL);
-    if(lpDispDev->cb >= offsetof(DISPLAY_DEVICEA, DeviceKey) + sizeof(lpDispDev->DeviceKey))
-        WideCharToMultiByte(CP_ACP, 0, ddW.DeviceKey, -1, lpDispDev->DeviceKey, sizeof(lpDispDev->DeviceKey), NULL, NULL);
-
-    return TRUE;
-}
-
-/***********************************************************************
- *		EnumDisplayDevicesW (USER32.@)
- */
-BOOL WINAPI EnumDisplayDevicesW( LPCWSTR lpDevice, DWORD i, LPDISPLAY_DEVICEW lpDisplayDevice,
-                                 DWORD dwFlags )
-{
-    FIXME("(%s,%d,%p,0x%08x), stub!\n",debugstr_w(lpDevice),i,lpDisplayDevice,dwFlags);
-
-    if (i)
-        return FALSE;
-
-    memcpy(lpDisplayDevice->DeviceName, primary_device_name, sizeof(primary_device_name));
-    memcpy(lpDisplayDevice->DeviceString, primary_device_string, sizeof(primary_device_string));
-  
-    lpDisplayDevice->StateFlags =
-        DISPLAY_DEVICE_ATTACHED_TO_DESKTOP |
-        DISPLAY_DEVICE_PRIMARY_DEVICE |
-        DISPLAY_DEVICE_VGA_COMPATIBLE;
-
-    if(lpDisplayDevice->cb >= offsetof(DISPLAY_DEVICEW, DeviceID) + sizeof(lpDisplayDevice->DeviceID))
-        memcpy(lpDisplayDevice->DeviceID, primary_device_deviceid, sizeof(primary_device_deviceid));
-    if(lpDisplayDevice->cb >= offsetof(DISPLAY_DEVICEW, DeviceKey) + sizeof(lpDisplayDevice->DeviceKey))
-        lpDisplayDevice->DeviceKey[0] = 0;
-
-    return TRUE;
-}
-
 /***********************************************************************
  *              QueryDisplayConfig (USER32.@)
  */
