@@ -494,6 +494,8 @@ static void test_Win32_Process( IWbemServices *services, BOOL use_full_path )
     IWbemClassObject *process, *sig_in, *out;
     IWbemQualifierSet *qualifiers;
     VARIANT retval, val;
+    SAFEARRAY *names;
+    LONG bound, i;
     DWORD full_path_len = 0;
     LONG flavor;
     CIMTYPE type;
@@ -518,6 +520,21 @@ static void test_Win32_Process( IWbemServices *services, BOOL use_full_path )
         win_skip( "Win32_Process not available\n" );
         return;
     }
+    names = NULL;
+    hr = IWbemClassObject_GetNames( process, NULL, 0, NULL, &names );
+    ok( hr == S_OK, "got %08x\n", hr );
+    ok( names != NULL, "names not set\n" );
+    hr = SafeArrayGetUBound( names, 1, &bound );
+    ok( hr == S_OK, "got %08x\n", hr );
+    for (i = 0; i <= bound; i++)
+    {
+        BSTR str;
+        hr = SafeArrayGetElement( names, &i, &str );
+        ok( hr == S_OK, "%d: got %08x\n", i, hr );
+        SysFreeString( str );
+    }
+    SafeArrayDestroy( names );
+
     sig_in = (void*)0xdeadbeef;
     hr = IWbemClassObject_GetMethod( process, getownerW, 0, &sig_in, NULL );
     ok( hr == S_OK, "failed to get GetOwner method %08x\n", hr );
@@ -1111,6 +1128,21 @@ static void test_GetNames( IWbemServices *services )
         IEnumWbemClassObject_Next( result, 10000, 1, &obj, &count );
         if (!count) break;
 
+        hr = IWbemClassObject_GetNames( obj, NULL, 0, NULL, NULL );
+        ok( hr == WBEM_E_INVALID_PARAMETER, "got %08x\n", hr );
+
+        hr = IWbemClassObject_GetNames( obj, NULL, 0, NULL, &names );
+        ok( hr == S_OK, "got %08x\n", hr );
+        SafeArrayDestroy( names );
+
+        hr = IWbemClassObject_GetNames( obj, NULL, WBEM_FLAG_ALWAYS, NULL, &names );
+        ok( hr == S_OK, "got %08x\n", hr );
+        SafeArrayDestroy( names );
+
+        hr = IWbemClassObject_GetNames( obj, NULL, WBEM_FLAG_ALWAYS | WBEM_MASK_CONDITION_ORIGIN, NULL, &names );
+        ok( hr == S_OK, "got %08x\n", hr );
+        SafeArrayDestroy( names );
+
         VariantInit( &val );
         hr = IWbemClassObject_GetNames( obj, NULL, WBEM_FLAG_NONSYSTEM_ONLY, &val, &names );
         ok( hr == S_OK, "got %08x\n", hr );
