@@ -22,7 +22,7 @@
 
 #include <stdarg.h>
 
-#ifdef HAVE_PNG_H
+#ifdef SONAME_LIBPNG
 #include <png.h>
 #endif
 
@@ -821,8 +821,7 @@ static HRESULT WINAPI PngDecoder_Initialize(IWICBitmapDecoder *iface, IStream *p
 end:
     LeaveCriticalSection(&This->lock);
 
-    if (row_pointers)
-        HeapFree(GetProcessHeap(), 0, row_pointers);
+    HeapFree(GetProcessHeap(), 0, row_pointers);
 
     return hr;
 }
@@ -2184,7 +2183,7 @@ HRESULT PngEncoder_CreateInstance(REFIID iid, void** ppv)
     return ret;
 }
 
-#else /* !HAVE_PNG_H */
+#else /* !SONAME_LIBPNG */
 
 HRESULT PngDecoder_CreateInstance(REFIID iid, void** ppv)
 {
