@@ -1055,6 +1055,11 @@ void WCMD_run_program (WCHAR *command, BOOL called)
   firstParam = WCMD_parameter(command, 0, NULL, FALSE, TRUE);
   if (!firstParam) return;
 
+  if (!firstParam[0]) {
+      errorlevel = 0;
+      return;
+  }
+
   /* Calculate the search path and stem to search for */
   if (wcspbrk (firstParam, delims) == NULL) {  /* No explicit path given, search path */
     static const WCHAR curDir[] = {'.',';','\0'};
@@ -1782,7 +1787,8 @@ static BOOL WCMD_IsEndQuote(const WCHAR *quote, int quoteIndex)
 
         /* Quote counting ends at EOL, redirection, space or pipe if current quote is complete */
         else if(((quoteCount % 2) == 0)
-            && ((quote[i] == '<') || (quote[i] == '>') || (quote[i] == '|') || (quote[i] == ' ')))
+            && ((quote[i] == '<') || (quote[i] == '>') || (quote[i] == '|') || (quote[i] == ' ') ||
+                (quote[i] == '&')))
         {
             break;
         }
